@@ -1,56 +1,26 @@
-#line 1
-##
-# name:      Pegex::Parser
-# abstract:  Pegex Parser Runtime
-# author:    Ingy döt Net <ingy@cpan.org>
-# license:   perl
-# copyright: 2011, 2012
-# see:
-# - Pegex::Grammar
-# - Pegex::Receiver
-
 package Pegex::Parser;
-use Pegex::Base;
 
 use Pegex::Input;
 
 use Scalar::Util;
 
-# Grammar object or class
+{
+    package Pegex::Constant;
+
+    our $Null = [];
+    our $Dummy = [];
+}
+
+package Pegex::Parser;
+
+use Pegex::Base;
+
 has grammar => (required => 1);
+has receiver => ();
+has input => ();
 
-# Receiver object or class
-has receiver => (
-    default => sub {
-        require Pegex::Receiver;
-        Pegex::Receiver->new();
-    },
-    lazy => 1,
-);
-
-#
-# Parser options
-#
-
-# Allow errors to not be thrown
-has throw_on_error => 1;
-
-# # Allow a partial parse
-# has 'partial' => 0;
-
-# Internal properties.
-has input => ();            # Input object to read from
-has buffer => ();           # Input buffer to parse
-has length => ();           # Length of buffer
-has error => ();            # Error message goes here
-has position => 0;          # Current position in buffer
-has farthest => 0;          # Farthest point matched in buffer
-has optimized => 0;         # Parser object has been optimized
-
-has rule => ();             # The current rule name.
-has parent => ();           # The grammar object pointing to the current rule.
-
-# Debug the parsing of input.
+has rule => ();
+has parent => ();
 has 'debug' => (
     default => sub {
         exists($ENV{PERL_PEGEX_DEBUG}) ? $ENV{PERL_PEGEX_DEBUG} :
@@ -59,57 +29,46 @@ has 'debug' => (
     },
 );
 
-sub BUILD {
-    my ($self) = @_;
-    my $grammar = $self->grammar;
-    my $receiver = $self->receiver;
-    if ($grammar and not ref $grammar) {
-        eval "require $grammar";
-        $self->{grammar} = $grammar->new;
-    }
-    if ($receiver and not ref $receiver) {
-        eval "require $receiver";
-        $self->{receiver} = $receiver->new;
-    }
-}
+has position => 0;
+has farthest => 0;
+has optimized => 0;
+
+has throw_on_error => 1;
 
 sub parse {
     my ($self, $input, $start) = @_;
-    $self->{position} = 0; # XXX Currently needed for repeated calls.
-
-    die 'Usage: ->parse($input [, $start_rule]'
-        unless 2 <= @_ and @_ <= 3;
-
-    $input = Pegex::Input->new(string => $input)
-        unless ref $input and UNIVERSAL::isa($input, 'Pegex::Input');
+    $self->{position} = 0;
 
+    if (not UNIVERSAL::isa($input, 'Pegex::Input')) {
+        $input = Pegex::Input->new(string => $input);
+    }
     $self->{input} = $input;
-
     $self->{input}->open unless $self->{input}{_is_open};
     $self->{buffer} = $self->{input}->read;
     $self->{length} = length ${$self->{buffer}};
 
-    my $grammar = $self->{grammar}
-        or die "No 'grammar'. Can't parse";
+    die "No 'grammar'. Can't parse" unless $self->{grammar};
 
-    my $tree = $self->{tree} = $grammar->{tree} //= $grammar->make_tree;
+    $self->{grammar}->{tree} = $self->{grammar}->make_tree
+        unless defined $self->{grammar}->{tree};
+    $self->{tree} = $self->{grammar}->{tree};
 
     my $start_rule_ref = $start ||
-        $tree->{'+toprule'} ||
-        ($tree->{'TOP'} ? 'TOP' : undef)
+        $self->{tree}->{'+toprule'} ||
+        ($self->{tree}->{'TOP'} ? 'TOP' : undef)
             or die "No starting rule for Pegex::Parser::parse";
 
-    $self->optimize_grammar($start_rule_ref);
+    die "No 'receiver'. Can't parse" unless $self->{receiver};
 
-    my $receiver = $self->{receiver}
-        or die "No 'receiver'. Can't parse";
+    $self->optimize_grammar($start_rule_ref);
 
     # Add circular ref and weaken it.
     $self->{receiver}{parser} = $self;
     Scalar::Util::weaken($self->{receiver}{parser});
 
     if ($self->{receiver}->can("initial")) {
-        @{$self}{'rule', 'parent'} = ($start_rule_ref, {});
+        $self->{rule} = $start_rule_ref;
+        $self->{parent} = {};
         $self->{receiver}->initial();
     }
 
@@ -123,7 +82,9 @@ sub parse {
     }
 
     if ($self->{receiver}->can("final")) {
-        @{$self}{'rule', 'parent'} = ($start_rule_ref, {});
+        $self->{rule} = $start_rule_ref;
+        $self->{parent} = {};
+        # XXX mismatch with ruby port
         $match = [ $self->{receiver}->final(@$match) ];
     }
 
@@ -132,34 +93,36 @@ sub parse {
 
 sub optimize_grammar {
     my ($self, $start) = @_;
-    return if $self->{optimized}++;
-    my $tree = $self->{tree};
-    for my $name (keys %$tree) {
-        my $node = $tree->{$name};
+    return if $self->{optimized};
+    while (my ($name, $node) = each %{$self->{tree}}) {
         next unless ref($node);
         $self->optimize_node($node);
     }
     $self->optimize_node({'.ref' => $start});
+    $self->{optimized} = 1;
 }
 
 sub optimize_node {
     my ($self, $node) = @_;
 
-    for (qw(ref rgx all any err code xxx)) {
-        die if $_ eq 'xxx';
-        if ($node->{rule} = $node->{".$_"}) {
-            $node->{kind} = $_;
-            $node->{method} = $self->can("match_$_") or die;
+    for my $kind (qw(ref rgx all any err code xxx)) {
+        die if $kind eq 'xxx';
+        if ($node->{rule} = $node->{".$kind"}) {
+            $node->{kind} = $kind;
+            $node->{method} = $self->can("match_$kind") or die;
             last;
         }
     }
 
     my ($min, $max) = @{$node}{'+min', '+max'};
-    $node->{'+min'} //= defined($max) ? 0 : 1;
-    $node->{'+max'} //= defined($min) ? 0 : 1;
-    $node->{'+asr'} //= 0;
+    $node->{'+min'} = defined($max) ? 0 : 1
+        unless defined $node->{'+min'};
+    $node->{'+max'} = defined($min) ? 0 : 1
+        unless defined $node->{'+max'};
+    $node->{'+asr'} = 0
+        unless defined $node->{'+asr'};
 
-    if ($node->{kind} =~ /(?:all|any)/) {
+    if ($node->{kind} =~ /^(?:all|any)$/) {
         $self->optimize_node($_) for @{$node->{rule}};
     }
     elsif ($node->{kind} eq 'ref') {
@@ -174,6 +137,9 @@ sub optimize_node {
         $node->{method} = $self->can("match_ref_trace")
             if $self->{debug};
     }
+    elsif ($node->{kind} eq 'rgx') {
+      # XXX $node;
+    }
     if (my $sep = $node->{'.sep'}) {
         $self->optimize_node($sep);
     }
@@ -199,16 +165,14 @@ sub match_next {
     }
     if ($max != 1) {
         $match = [$match];
-        if (($self->{position} = $position) > $self->{farthest}) {
-            $self->{farthest} = $position;
-        }
+        $self->{farthest} = $position
+            if ($self->{position} = $position) > $self->{farthest};
     }
-    my $result = (($count >= $min and (not $max or $count <= $max)) ? 1 : 0)
+    my $result = ($count >= $min and (not $max or $count <= $max))
         ^ ($assertion == -1);
     if (not($result) or $assertion) {
-        if (($self->{position} = $position) > $self->{farthest}) {
-            $self->{farthest} = $position;
-        }
+        $self->{farthest} = $position
+            if ($self->{position} = $position) > $self->{farthest};
     }
 
     return ($result ? $next->{'-skip'} ? [] : $match : 0);
@@ -228,34 +192,26 @@ sub match_next_with_sep {
         $count++;
         push @$match, @$return;
         $return = $self->match_next($sep) or last;
-        my @return = @$return;
-        if (@return) {
-            @return = @{$return[0]} if $smax != 1;
-            push @$match, @return;
-        }
+        push @$match, $smax == 1 ? @$return : @{$return->[0]} if @$return;
         $scount++;
     }
-    if ($max != 1) {
-        $match = [$match];
-    }
-    my $result = (($count >= $min and (not $max or $count <= $max)) ? 1 : 0);
+    $match = [$match] if $max != 1;
+    my $result = ($count >= $min and ($max == 0 or $count <= $max));
     if ($count == $scount and not $sep->{'+eok'}) {
-        if (($self->{position} = $position) > $self->{farthest}) {
-            $self->{farthest} = $position;
-        }
+        $self->{farthest} = $position
+            if ($self->{position} = $position) > $self->{farthest};
     }
 
     return ($result ? $next->{'-skip'} ? [] : $match : 0);
 }
 
-my $dummy = [1];
 sub match_ref {
-    my ($self) = @_; # $self, $ref, $parent
-    my $rule = $self->{tree}{$_[1]};
-
+    my ($self, $ref, $parent) = @_;
+    my $rule = $self->{tree}{$ref};
     my $match = $self->match_next($rule) or return 0;
-    return $dummy unless $rule->{action};
-    @{$self}{'rule', 'parent'} = @_[1,2];
+    return $Pegex::Constant::Dummy unless $rule->{action};
+    @{$self}{'rule', 'parent'} = ($ref, $parent);
+    # XXX API mismatch
     [ $rule->{action}->($self->{receiver}, @$match) ];
 }
 
@@ -263,19 +219,16 @@ sub match_rgx {
     my ($self, $regexp) = @_;
     my $buffer = $self->{buffer};
 
-    my $position = pos($$buffer) = $self->{position};
+    pos($$buffer) = $self->{position};
 
     $$buffer =~ /$regexp/g or return 0;
-    $position = pos($$buffer);
+    $self->{position} = pos($$buffer);
 
     no strict 'refs';
     my $match = [ map $$_, 1..$#+ ];
     $match = [ $match ] if $#+ > 1;
-
-    if (($self->{position} = $position) > $self->{farthest}) {
-        $self->{farthest} = $position;
-    }
-
+    $self->{farthest} = $self->{position}
+        if $self->{position} > $self->{farthest};
     return $match;
 }
 
@@ -286,14 +239,14 @@ sub match_all {
     my $len = 0;
     for my $elem (@$list) {
         if (my $match = $self->match_next($elem)) {
-            next if $elem->{'+asr'} or $elem->{'-skip'};
-            push @$set, @$match;
-            $len++;
+            if (not ($elem->{'+asr'} or $elem->{'-skip'})) {
+                push @$set, @$match;
+                $len++;
+            }
         }
         else {
-            if (($self->{position} = $position) > $self->{farthest}) {
-                $self->{farthest} = $position;
-            }
+            $self->{farthest} = $position
+                if ($self->{position} = $position) > $self->{farthest};
             return 0;
         }
     }
@@ -316,11 +269,12 @@ sub match_err {
     $self->throw_error($error);
 }
 
-sub match_code {
-    my ($self, $code) = @_;
-    my $method = "match_rule_$code";
-    return $self->$method();
-}
+# TODO not supported yet
+# sub match_code {
+#     my ($self, $code) = @_;
+#     my $method = "match_rule_$code";
+#     return $self->$method();
+# }
 
 sub match_ref_trace {
     my ($self, $ref) = @_;
@@ -353,8 +307,8 @@ sub trace {
 
 sub throw_error {
     my ($self, $msg) = @_;
-    $self->format_error($msg);
-    return 0 unless $self->{throw_on_error};
+    $@ = $self->{error} = $self->format_error($msg);
+    return undef unless $self->{throw_on_error};
     require Carp;
     Carp::croak($self->{error});
 }
@@ -377,7 +331,7 @@ sub format_error {
     $pretext =~ s/.*\n//gs;
     $context =~ s/\n/\\n/g;
 
-    $@ = $self->{error} = <<"...";
+    return <<"...";
 Error parsing Pegex document:
   msg:      $msg
   line:     $line
@@ -389,4 +343,3 @@ Error parsing Pegex document:
 }
 
 1;
-
